"""
节日提醒， festival reminder
"""
import json
import datetime
import time

import requests
import schedule


def bot(festival_info, days_count, date_str):
    """
    机器人
    """
    level = festival_info["level"]
    festival = festival_info["festival"]
    category = festival_info["category"]
    if festival == "龙年龙月龙日龙时":
        date_str = "\n" + date_str + " 07:00-09:00"
    # url = "https://open.feishu.cn/open-apis/bot/v2/hook/8eda2f73-e6f5-4fcd-944e-5a34579e1049"
    url = 'https://open.feishu.cn/open-apis/bot/v2/hook/92045725-d126-465b-babe-7e2a0aa048b3'
    headers = {"Content-Type": "application/json"}

    if level == "P0":
        payload = {
            "msg_type": "interactive",
            "card": {
                "elements": [
                    {
                        "tag": "div",
                        "text": {
                            "content": "📣热点内容节点通知，请注意查收<at id=all></at>\n",
                            "tag": "lark_md"
                        }
                    },
                    {
                        "tag": "div",
                        "text": {
                            "content": "节日类型: **{}**\n距离**{}**还有**{}天**，请提前做好内容准备哦\n**{}日期: \t{}**".format(
                                category, festival, days_count, festival, date_str
                            ),
                            "tag": "lark_md",
                        },
                    },
                ],
                "header": {"title": {"content": "【🔥P0重点关注】", "tag": "plain_text"}},
            },
        }
        res = requests.request("POST", url=url, headers=headers, data=json.dumps(payload))
        print(res.text)
    if level == "P1":
        payload = {
            "msg_type": "interactive",
            "card": {
                "elements": [
                    {
                        "tag": "div",
                        "text": {
                            "content": "📣热点内容节点通知，请注意查收<at id=all></at>\n",
                            "tag": "lark_md"
                        }
                    },
                    {
                        "tag": "div",
                        "text": {
                            "content": "节日类型: **{}**\n距离**{}**还有**{}天**，请提前做好内容准备哦\n**{}日期: \t{}**".format(
                                category, festival, days_count, festival, date_str
                            ),
                            "tag": "lark_md",
                        },
                    },
                ],
                "header": {"title": {"content": "【💡P1关注】", "tag": "plain_text"}},
            },
        }
        res = requests.request("POST", url=url, headers=headers, data=json.dumps(payload))
        print(res.text)


class FestivalReminder(object):
    """
    节日提醒对象
    """

    def __init__(self):
        self.festival_dict = {
            "2024": {
                "2024-01-01": [
                    {
                        "level": "P0",
                        "festival": "元旦",
                        "category": "节假日"
                    }
                ],
                "2024-01-18": [
                    {
                        "level": "P0",
                        "festival": "腊八节",
                        "category": "节假日"
                    }
                ],
                "2024-02-02": [
                    {
                        "level": "P0",
                        "festival": "小年",
                        "category": "节假日"
                    }
                ],
                "2024-03-12": [
                    {
                        "level": "P1",
                        "festival": "植树节",
                        "category": "节假日"
                    },
                    {
                        "level": "P1",
                        "festival": "植树节",
                        "category": "节假日"
                    },
                    {
                        "level": "P1",
                        "festival": "孙中山逝世纪念日",
                        "category": "纪念日"
                    }
                ],
                "2024-02-03": [
                    {
                        "level": "P0",
                        "festival": "小年",
                        "category": "节假日"
                    }
                ],
                "2024-02-09": [
                    {
                        "level": "P0",
                        "festival": "除夕",
                        "category": "节假日"
                    }
                ],
                "2024-02-10": [
                    {
                        "level": "P0",
                        "festival": "春节",
                        "category": "节假日"
                    },
                    {
                        "level": "P0",
                        "festival": "正月初一",
                        "category": "节假日"
                    }
                ],
                "2024-02-11": [
                    {
                        "level": "P0",
                        "festival": "正月初二",
                        "category": "节假日"
                    }
                ],
                "2024-02-12": [
                    {
                        "level": "P0",
                        "festival": "正月初三",
                        "category": "节假日"
                    }
                ],
                "2024-02-13": [
                    {
                        "level": "P0",
                        "festival": "正月初四",
                        "category": "节假日"
                    }
                ],
                "2024-02-14": [
                    {
                        "level": "P0",
                        "festival": "正月初五",
                        "category": "节假日"
                    },
                    {
                        "level": "P1",
                        "festival": "情人节",
                        "category": "节假日"
                    }
                ],
                "2024-02-24": [
                    {
                        "level": "P0",
                        "festival": "元宵节",
                        "category": "节假日"
                    }
                ],
                "2024-03-11": [
                    {
                        "level": "P0",
                        "festival": "龙抬头",
                        "category": "节假日"
                    }
                ],
                "2024-03-08": [
                    {
                        "level": "P1",
                        "festival": "妇女节",
                        "category": "节假日"
                    }
                ],
                "2024-05-01": [
                    {
                        "level": "P0",
                        "festival": "劳动节",
                        "category": "节假日"
                    }
                ],
                "2024-05-12": [
                    {
                        "level": "P0",
                        "festival": "母亲节",
                        "category": "节假日"
                    }
                ],
                "2024-06-01": [
                    {
                        "level": "P1",
                        "festival": "儿童节",
                        "category": "节假日"
                    }
                ],
                "2024-06-10": [
                    {
                        "level": "P0",
                        "festival": "端午节",
                        "category": "节假日"
                    }
                ],
                "2024-06-16": [
                    {
                        "level": "P0",
                        "festival": "父亲节",
                        "category": "节假日"
                    }
                ],
                "2024-07-01": [
                    {
                        "level": "P0",
                        "festival": "建党节",
                        "category": "节假日"
                    }
                ],
                "2024-08-01": [
                    {
                        "level": "P1",
                        "festival": "建军节",
                        "category": "节假日"
                    }
                ],
                "2024-08-10": [
                    {
                        "level": "P0",
                        "festival": "七夕节",
                        "category": "节假日"
                    }
                ],
                "2024-08-18": [
                    {
                        "level": "P1",
                        "festival": "中元节",
                        "category": "节假日"
                    }
                ],
                "2024-09-17": [
                    {
                        "level": "P0",
                        "festival": "中秋节",
                        "category": "节假日"
                    }
                ],
                "2024-10-01": [
                    {
                        "level": "P0",
                        "festival": "国庆节",
                        "category": "节假日"
                    }
                ],
                "2024-10-11": [
                    {
                        "level": "P0",
                        "festival": "重阳节",
                        "category": "节假日"
                    }
                ],
                "2024-11-28": [
                    {
                        "level": "P1",
                        "festival": "感恩节",
                        "category": "节假日"
                    }
                ],
                "2024-11-27": [
                    {
                        "level": "P1",
                        "festival": "感恩节",
                        "category": "节假日"
                    }
                ],
                "2024-12-13": [
                    {
                        "level": "P0",
                        "festival": "公祭日",
                        "category": "节假日"
                    }
                ],
                "2024-12-24": [
                    {
                        "level": "P1",
                        "festival": "平安夜",
                        "category": "节假日"
                    }
                ],
                "2024-12-25": [
                    {
                        "level": "P1",
                        "festival": "圣诞节",
                        "category": "节假日"
                    }
                ],
                "2024-01-06": [
                    {
                        "level": "P1",
                        "festival": "小寒",
                        "category": "节气"
                    }
                ],
                "2024-01-20": [
                    {
                        "level": "P1",
                        "festival": "大寒",
                        "category": "节气"
                    }
                ],
                "2024-02-04": [
                    {
                        "level": "P0",
                        "festival": "立春",
                        "category": "节气"
                    }
                ],
                "2024-02-19": [
                    {
                        "level": "P1",
                        "festival": "雨水",
                        "category": "节气"
                    },
                    {
                        "level": "P0",
                        "festival": "邓小平逝世纪念日",
                        "category": "纪念日"
                    }
                ],
                "2024-03-05": [
                    {
                        "level": "P1",
                        "festival": "惊蛰",
                        "category": "节气"
                    },
                    {
                        "level": "P0",
                        "festival": "周恩来诞辰纪念日",
                        "category": "纪念日"
                    },
                    {
                        "level": "P1",
                        "festival": "学雷锋纪念日纪念日",
                        "category": "纪念日"
                    },
                    {
                        "level": "P0",
                        "festival": "两会",
                        "category": "热点"
                    },
                    {
                        "level": "P0",
                        "festival": "两会",
                        "category": "热点"
                    },
                    {
                        "level": "P0",
                        "festival": "两会",
                        "category": "热点"
                    }
                ],
                "2024-03-20": [
                    {
                        "level": "P1",
                        "festival": "春分",
                        "category": "节气"
                    }
                ],
                "2024-04-04": [
                    {
                        "level": "P0",
                        "festival": "清明",
                        "category": "节气"
                    }
                ],
                "2024-04-19": [
                    {
                        "level": "P1",
                        "festival": "谷雨",
                        "category": "节气"
                    }
                ],
                "2024-04-22": [
                    {
                        "level": "P1",
                        "festival": "龙年龙月龙日龙时",
                        "category": "热点"
                    }
                ],
                "2024-05-04": [
                    {
                        "level": "P1",
                        "festival": "龙年龙月龙日龙时",
                        "category": "热点"
                    }
                ],
                "2024-05-05": [
                    {
                        "level": "P0",
                        "festival": "立夏",
                        "category": "节气"
                    }
                ],
                "2024-05-20": [
                    {
                        "level": "P1",
                        "festival": "小满",
                        "category": "节气"
                    }
                ],
                "2024-06-05": [
                    {
                        "level": "P1",
                        "festival": "芒种",
                        "category": "节气"
                    }
                ],
                "2024-06-21": [
                    {
                        "level": "P1",
                        "festival": "夏至",
                        "category": "节气"
                    }
                ],
                "2024-07-06": [
                    {
                        "level": "P1",
                        "festival": "小暑",
                        "category": "节气"
                    },
                    {
                        "level": "P1",
                        "festival": "朱德逝世纪念日",
                        "category": "纪念日"
                    }
                ],
                "2024-07-22": [
                    {
                        "level": "P1",
                        "festival": "大暑",
                        "category": "节气"
                    }
                ],
                "2024-08-07": [
                    {
                        "level": "P0",
                        "festival": "立秋",
                        "category": "节气"
                    }
                ],
                "2024-08-22": [
                    {
                        "level": "P1",
                        "festival": "处暑",
                        "category": "节气"
                    },
                    {
                        "level": "P0",
                        "festival": "邓小平诞辰纪念日",
                        "category": "纪念日"
                    }
                ],
                "2024-09-07": [
                    {
                        "level": "P1",
                        "festival": "白露",
                        "category": "节气"
                    },
                    {
                        "level": "P1",
                        "festival": "袁隆平诞辰纪念日",
                        "category": "纪念日"
                    }
                ],
                "2024-09-22": [
                    {
                        "level": "P1",
                        "festival": "秋分",
                        "category": "节气"
                    }
                ],
                "2024-10-08": [
                    {
                        "level": "P1",
                        "festival": "寒露",
                        "category": "节气"
                    }
                ],
                "2024-10-23": [
                    {
                        "level": "P1",
                        "festival": "霜降",
                        "category": "节气"
                    }
                ],
                "2024-11-07": [
                    {
                        "level": "P0",
                        "festival": "立冬",
                        "category": "节气"
                    }
                ],
                "2024-11-22": [
                    {
                        "level": "P1",
                        "festival": "小雪",
                        "category": "节气"
                    }
                ],
                "2024-12-06": [
                    {
                        "level": "P1",
                        "festival": "大雪",
                        "category": "节气"
                    }
                ],
                "2024-12-21": [
                    {
                        "level": "P0",
                        "festival": "冬至",
                        "category": "节气"
                    }
                ],
                "2024-11-12": [
                    {
                        "level": "P1",
                        "festival": "孙中山诞辰纪念日",
                        "category": "纪念日"
                    }
                ],
                "2024-12-26": [
                    {
                        "level": "P0",
                        "festival": "毛主席诞辰纪念日",
                        "category": "纪念日"
                    }
                ],
                "2024-09-09": [
                    {
                        "level": "P0",
                        "festival": "毛主席逝世纪念日",
                        "category": "纪念日"
                    }
                ],
                "2024-01-08": [
                    {
                        "level": "P0",
                        "festival": "周恩来逝世纪念日",
                        "category": "纪念日"
                    }
                ],
                "2024-07-03": [
                    {
                        "level": "P0",
                        "festival": "李克强诞辰纪念日",
                        "category": "纪念日"
                    }
                ],
                "2024-10-27": [
                    {
                        "level": "P0",
                        "festival": "李克强逝世纪念日",
                        "category": "纪念日"
                    },
                    {
                        "level": "P1",
                        "festival": "吴尊友逝世纪念日",
                        "category": "纪念日"
                    }
                ],
                "2024-09-18": [
                    {
                        "level": "P0",
                        "festival": "九一八纪念日",
                        "category": "纪念日"
                    }
                ],
                "2024-07-07": [
                    {
                        "level": "P0",
                        "festival": "七七事变纪念日",
                        "category": "纪念日"
                    }
                ],
                "2024-05-22": [
                    {
                        "level": "P1",
                        "festival": "袁隆平逝世纪念日",
                        "category": "纪念日"
                    }
                ],
                "2024-10-24": [
                    {
                        "level": "P1",
                        "festival": "彭德怀诞辰纪念日",
                        "category": "纪念日"
                    }
                ],
                "2024-11-29": [
                    {
                        "level": "P1",
                        "festival": "彭德怀逝世纪念日",
                        "category": "纪念日"
                    }
                ],
                "2024-12-01": [
                    {
                        "level": "P1",
                        "festival": "朱德诞辰纪念日",
                        "category": "纪念日"
                    }
                ],
                "2024-03-15": [
                    {
                        "level": "P1",
                        "festival": "315消费者权益保护日",
                        "category": "热点"
                    },
                    {
                        "level": "P1",
                        "festival": "315消费者权益保护日",
                        "category": "热点"
                    },
                    {
                        "level": "P1",
                        "festival": "315消费者权益保护日",
                        "category": "热点"
                    }
                ]
            },
            "2025": {
                "2025-01-01": [
                    {
                        "level": "P0",
                        "festival": "元旦",
                        "category": "节假日"
                    }
                ],
                "2025-01-07": [
                    {
                        "level": "P0",
                        "festival": "腊八节",
                        "category": "节假日"
                    }
                ],
                "2025-01-22": [
                    {
                        "level": "P0",
                        "festival": "小年",
                        "category": "节假日"
                    }
                ],
                "2025-03-12": [
                    {
                        "level": "P1",
                        "festival": "植树节",
                        "category": "节假日"
                    },
                    {
                        "level": "P1",
                        "festival": "植树节",
                        "category": "节假日"
                    },
                    {
                        "level": "P1",
                        "festival": "孙中山逝世纪念日",
                        "category": "纪念日"
                    }
                ],
                "2025-01-23": [
                    {
                        "level": "P0",
                        "festival": "小年",
                        "category": "节假日"
                    }
                ],
                "2025-01-28": [
                    {
                        "level": "P0",
                        "festival": "除夕",
                        "category": "节假日"
                    }
                ],
                "2025-01-29": [
                    {
                        "level": "P0",
                        "festival": "春节",
                        "category": "节假日"
                    },
                    {
                        "level": "P0",
                        "festival": "正月初一",
                        "category": "节假日"
                    }
                ],
                "2025-01-30": [
                    {
                        "level": "P0",
                        "festival": "正月初二",
                        "category": "节假日"
                    }
                ],
                "2025-01-31": [
                    {
                        "level": "P0",
                        "festival": "正月初三",
                        "category": "节假日"
                    }
                ],
                "2025-02-01": [
                    {
                        "level": "P0",
                        "festival": "正月初四",
                        "category": "节假日"
                    }
                ],
                "2025-02-02": [
                    {
                        "level": "P0",
                        "festival": "正月初五",
                        "category": "节假日"
                    }
                ],
                "2025-02-14": [
                    {
                        "level": "P1",
                        "festival": "情人节",
                        "category": "节假日"
                    }
                ],
                "2025-02-12": [
                    {
                        "level": "P0",
                        "festival": "元宵节",
                        "category": "节假日"
                    }
                ],
                "2025-03-01": [
                    {
                        "level": "P0",
                        "festival": "龙抬头",
                        "category": "节假日"
                    }
                ],
                "2025-03-08": [
                    {
                        "level": "P1",
                        "festival": "妇女节",
                        "category": "节假日"
                    }
                ],
                "2025-05-01": [
                    {
                        "level": "P0",
                        "festival": "劳动节",
                        "category": "节假日"
                    }
                ],
                "2025-05-11": [
                    {
                        "level": "P0",
                        "festival": "母亲节",
                        "category": "节假日"
                    }
                ],
                "2025-06-01": [
                    {
                        "level": "P1",
                        "festival": "儿童节",
                        "category": "节假日"
                    }
                ],
                "2025-05-31": [
                    {
                        "level": "P0",
                        "festival": "端午节",
                        "category": "节假日"
                    }
                ],
                "2025-06-15": [
                    {
                        "level": "P0",
                        "festival": "父亲节",
                        "category": "节假日"
                    }
                ],
                "2025-07-01": [
                    {
                        "level": "P0",
                        "festival": "建党节",
                        "category": "节假日"
                    }
                ],
                "2025-08-01": [
                    {
                        "level": "P1",
                        "festival": "建军节",
                        "category": "节假日"
                    }
                ],
                "2025-08-29": [
                    {
                        "level": "P0",
                        "festival": "七夕节",
                        "category": "节假日"
                    }
                ],
                "2025-09-06": [
                    {
                        "level": "P1",
                        "festival": "中元节",
                        "category": "节假日"
                    }
                ],
                "2025-10-06": [
                    {
                        "level": "P0",
                        "festival": "中秋节",
                        "category": "节假日"
                    }
                ],
                "2025-10-01": [
                    {
                        "level": "P0",
                        "festival": "国庆节",
                        "category": "节假日"
                    }
                ],
                "2025-10-29": [
                    {
                        "level": "P0",
                        "festival": "重阳节",
                        "category": "节假日"
                    }
                ],
                "2025-12-13": [
                    {
                        "level": "P0",
                        "festival": "公祭日",
                        "category": "节假日"
                    }
                ],
                "2025-12-24": [
                    {
                        "level": "P1",
                        "festival": "平安夜",
                        "category": "节假日"
                    }
                ],
                "2025-12-25": [
                    {
                        "level": "P1",
                        "festival": "圣诞节",
                        "category": "节假日"
                    }
                ],
                "2025-01-05": [
                    {
                        "level": "P1",
                        "festival": "小寒",
                        "category": "节气"
                    }
                ],
                "2025-01-20": [
                    {
                        "level": "P1",
                        "festival": "大寒",
                        "category": "节气"
                    }
                ],
                "2025-02-03": [
                    {
                        "level": "P0",
                        "festival": "立春",
                        "category": "节气"
                    }
                ],
                "2025-02-18": [
                    {
                        "level": "P1",
                        "festival": "雨水",
                        "category": "节气"
                    }
                ],
                "2025-03-05": [
                    {
                        "level": "P1",
                        "festival": "惊蛰",
                        "category": "节气"
                    },
                    {
                        "level": "P0",
                        "festival": "周恩来诞辰纪念日",
                        "category": "纪念日"
                    },
                    {
                        "level": "P1",
                        "festival": "学雷锋纪念日纪念日",
                        "category": "纪念日"
                    }
                ],
                "2025-03-20": [
                    {
                        "level": "P1",
                        "festival": "春分",
                        "category": "节气"
                    }
                ],
                "2025-04-04": [
                    {
                        "level": "P0",
                        "festival": "清明",
                        "category": "节气"
                    }
                ],
                "2025-04-20": [
                    {
                        "level": "P1",
                        "festival": "谷雨",
                        "category": "节气"
                    }
                ],
                "2025-05-05": [
                    {
                        "level": "P0",
                        "festival": "立夏",
                        "category": "节气"
                    }
                ],
                "2025-05-21": [
                    {
                        "level": "P1",
                        "festival": "小满",
                        "category": "节气"
                    }
                ],
                "2025-06-05": [
                    {
                        "level": "P1",
                        "festival": "芒种",
                        "category": "节气"
                    }
                ],
                "2025-06-21": [
                    {
                        "level": "P1",
                        "festival": "夏至",
                        "category": "节气"
                    }
                ],
                "2025-07-07": [
                    {
                        "level": "P1",
                        "festival": "小暑",
                        "category": "节气"
                    },
                    {
                        "level": "P0",
                        "festival": "七七事变纪念日",
                        "category": "纪念日"
                    }
                ],
                "2025-07-22": [
                    {
                        "level": "P1",
                        "festival": "大暑",
                        "category": "节气"
                    }
                ],
                "2025-08-07": [
                    {
                        "level": "P0",
                        "festival": "立秋",
                        "category": "节气"
                    }
                ],
                "2025-08-23": [
                    {
                        "level": "P1",
                        "festival": "处暑",
                        "category": "节气"
                    }
                ],
                "2025-09-07": [
                    {
                        "level": "P1",
                        "festival": "白露",
                        "category": "节气"
                    },
                    {
                        "level": "P1",
                        "festival": "袁隆平诞辰纪念日",
                        "category": "纪念日"
                    }
                ],
                "2025-09-23": [
                    {
                        "level": "P1",
                        "festival": "秋分",
                        "category": "节气"
                    }
                ],
                "2025-10-08": [
                    {
                        "level": "P1",
                        "festival": "寒露",
                        "category": "节气"
                    }
                ],
                "2025-10-23": [
                    {
                        "level": "P1",
                        "festival": "霜降",
                        "category": "节气"
                    }
                ],
                "2025-11-07": [
                    {
                        "level": "P0",
                        "festival": "立冬",
                        "category": "节气"
                    }
                ],
                "2025-11-22": [
                    {
                        "level": "P1",
                        "festival": "小雪",
                        "category": "节气"
                    }
                ],
                "2025-12-07": [
                    {
                        "level": "P1",
                        "festival": "大雪",
                        "category": "节气"
                    }
                ],
                "2025-12-21": [
                    {
                        "level": "P0",
                        "festival": "冬至",
                        "category": "节气"
                    }
                ],
                "2025-11-12": [
                    {
                        "level": "P1",
                        "festival": "孙中山诞辰纪念日",
                        "category": "纪念日"
                    }
                ],
                "2025-12-26": [
                    {
                        "level": "P0",
                        "festival": "毛主席诞辰纪念日",
                        "category": "纪念日"
                    }
                ],
                "2025-09-09": [
                    {
                        "level": "P0",
                        "festival": "毛主席逝世纪念日",
                        "category": "纪念日"
                    }
                ],
                "2025-01-08": [
                    {
                        "level": "P0",
                        "festival": "周恩来逝世纪念日",
                        "category": "纪念日"
                    }
                ],
                "2025-08-22": [
                    {
                        "level": "P0",
                        "festival": "邓小平诞辰纪念日",
                        "category": "纪念日"
                    }
                ],
                "2025-02-19": [
                    {
                        "level": "P0",
                        "festival": "邓小平逝世纪念日",
                        "category": "纪念日"
                    }
                ],
                "2025-07-03": [
                    {
                        "level": "P0",
                        "festival": "李克强诞辰纪念日",
                        "category": "纪念日"
                    }
                ],
                "2025-10-27": [
                    {
                        "level": "P0",
                        "festival": "李克强逝世纪念日",
                        "category": "纪念日"
                    },
                    {
                        "level": "P1",
                        "festival": "吴尊友逝世纪念日",
                        "category": "纪念日"
                    }
                ],
                "2025-09-18": [
                    {
                        "level": "P0",
                        "festival": "九一八纪念日",
                        "category": "纪念日"
                    }
                ],
                "2025-05-22": [
                    {
                        "level": "P1",
                        "festival": "袁隆平逝世纪念日",
                        "category": "纪念日"
                    }
                ],
                "2025-10-24": [
                    {
                        "level": "P1",
                        "festival": "彭德怀诞辰纪念日",
                        "category": "纪念日"
                    }
                ],
                "2025-11-29": [
                    {
                        "level": "P1",
                        "festival": "彭德怀逝世纪念日",
                        "category": "纪念日"
                    }
                ],
                "2025-12-01": [
                    {
                        "level": "P1",
                        "festival": "朱德诞辰纪念日",
                        "category": "纪念日"
                    }
                ],
                "2025-07-06": [
                    {
                        "level": "P1",
                        "festival": "朱德逝世纪念日",
                        "category": "纪念日"
                    }
                ]
            },
            "2026": {
                "2026-01-01": [
                    {
                        "level": "P0",
                        "festival": "元旦",
                        "category": "节假日"
                    }
                ],
                "2026-01-26": [
                    {
                        "level": "P0",
                        "festival": "腊八节",
                        "category": "节假日"
                    }
                ],
                "2026-02-10": [
                    {
                        "level": "P0",
                        "festival": "小年",
                        "category": "节假日"
                    }
                ],
                "2026-03-12": [
                    {
                        "level": "P1",
                        "festival": "植树节",
                        "category": "节假日"
                    },
                    {
                        "level": "P1",
                        "festival": "植树节",
                        "category": "节假日"
                    },
                    {
                        "level": "P1",
                        "festival": "孙中山逝世纪念日",
                        "category": "纪念日"
                    }
                ],
                "2026-02-11": [
                    {
                        "level": "P0",
                        "festival": "小年",
                        "category": "节假日"
                    }
                ],
                "2026-02-16": [
                    {
                        "level": "P0",
                        "festival": "除夕",
                        "category": "节假日"
                    }
                ],
                "2026-02-17": [
                    {
                        "level": "P0",
                        "festival": "春节",
                        "category": "节假日"
                    },
                    {
                        "level": "P0",
                        "festival": "正月初一",
                        "category": "节假日"
                    }
                ],
                "2026-02-18": [
                    {
                        "level": "P0",
                        "festival": "正月初二",
                        "category": "节假日"
                    },
                    {
                        "level": "P1",
                        "festival": "雨水",
                        "category": "节气"
                    }
                ],
                "2026-02-19": [
                    {
                        "level": "P0",
                        "festival": "正月初三",
                        "category": "节假日"
                    },
                    {
                        "level": "P0",
                        "festival": "邓小平逝世纪念日",
                        "category": "纪念日"
                    }
                ],
                "2026-02-20": [
                    {
                        "level": "P0",
                        "festival": "正月初四",
                        "category": "节假日"
                    }
                ],
                "2026-02-21": [
                    {
                        "level": "P0",
                        "festival": "正月初五",
                        "category": "节假日"
                    }
                ],
                "2026-02-14": [
                    {
                        "level": "P1",
                        "festival": "情人节",
                        "category": "节假日"
                    }
                ],
                "2026-03-03": [
                    {
                        "level": "P0",
                        "festival": "元宵节",
                        "category": "节假日"
                    }
                ],
                "2026-03-20": [
                    {
                        "level": "P0",
                        "festival": "龙抬头",
                        "category": "节假日"
                    },
                    {
                        "level": "P1",
                        "festival": "春分",
                        "category": "节气"
                    }
                ],
                "2026-03-08": [
                    {
                        "level": "P1",
                        "festival": "妇女节",
                        "category": "节假日"
                    }
                ],
                "2026-05-01": [
                    {
                        "level": "P0",
                        "festival": "劳动节",
                        "category": "节假日"
                    }
                ],
                "2026-05-10": [
                    {
                        "level": "P0",
                        "festival": "母亲节",
                        "category": "节假日"
                    }
                ],
                "2026-06-01": [
                    {
                        "level": "P1",
                        "festival": "儿童节",
                        "category": "节假日"
                    }
                ],
                "2026-06-19": [
                    {
                        "level": "P0",
                        "festival": "端午节",
                        "category": "节假日"
                    }
                ],
                "2026-06-21": [
                    {
                        "level": "P0",
                        "festival": "父亲节",
                        "category": "节假日"
                    },
                    {
                        "level": "P1",
                        "festival": "夏至",
                        "category": "节气"
                    }
                ],
                "2026-07-01": [
                    {
                        "level": "P0",
                        "festival": "建党节",
                        "category": "节假日"
                    }
                ],
                "2026-08-01": [
                    {
                        "level": "P1",
                        "festival": "建军节",
                        "category": "节假日"
                    }
                ],
                "2026-08-19": [
                    {
                        "level": "P0",
                        "festival": "七夕节",
                        "category": "节假日"
                    }
                ],
                "2026-08-27": [
                    {
                        "level": "P1",
                        "festival": "中元节",
                        "category": "节假日"
                    }
                ],
                "2026-09-25": [
                    {
                        "level": "P0",
                        "festival": "中秋节",
                        "category": "节假日"
                    }
                ],
                "2026-10-01": [
                    {
                        "level": "P0",
                        "festival": "国庆节",
                        "category": "节假日"
                    }
                ],
                "2026-10-18": [
                    {
                        "level": "P0",
                        "festival": "重阳节",
                        "category": "节假日"
                    }
                ],
                "2026-11-26": [
                    {
                        "level": "P1",
                        "festival": "感恩节",
                        "category": "节假日"
                    }
                ],
                "2026-12-13": [
                    {
                        "level": "P0",
                        "festival": "公祭日",
                        "category": "节假日"
                    }
                ],
                "2026-12-24": [
                    {
                        "level": "P1",
                        "festival": "平安夜",
                        "category": "节假日"
                    }
                ],
                "2026-12-25": [
                    {
                        "level": "P1",
                        "festival": "圣诞节",
                        "category": "节假日"
                    }
                ],
                "2026-01-05": [
                    {
                        "level": "P1",
                        "festival": "小寒",
                        "category": "节气"
                    }
                ],
                "2026-01-20": [
                    {
                        "level": "P1",
                        "festival": "大寒",
                        "category": "节气"
                    }
                ],
                "2026-02-04": [
                    {
                        "level": "P0",
                        "festival": "立春",
                        "category": "节气"
                    }
                ],
                "2026-03-05": [
                    {
                        "level": "P1",
                        "festival": "惊蛰",
                        "category": "节气"
                    },
                    {
                        "level": "P0",
                        "festival": "周恩来诞辰纪念日",
                        "category": "纪念日"
                    },
                    {
                        "level": "P1",
                        "festival": "学雷锋纪念日纪念日",
                        "category": "纪念日"
                    }
                ],
                "2026-04-05": [
                    {
                        "level": "P0",
                        "festival": "清明",
                        "category": "节气"
                    }
                ],
                "2026-04-20": [
                    {
                        "level": "P1",
                        "festival": "谷雨",
                        "category": "节气"
                    }
                ],
                "2026-05-05": [
                    {
                        "level": "P0",
                        "festival": "立夏",
                        "category": "节气"
                    }
                ],
                "2026-05-21": [
                    {
                        "level": "P1",
                        "festival": "小满",
                        "category": "节气"
                    }
                ],
                "2026-06-05": [
                    {
                        "level": "P1",
                        "festival": "芒种",
                        "category": "节气"
                    }
                ],
                "2026-07-07": [
                    {
                        "level": "P1",
                        "festival": "小暑",
                        "category": "节气"
                    },
                    {
                        "level": "P0",
                        "festival": "七七事变纪念日",
                        "category": "纪念日"
                    }
                ],
                "2026-07-23": [
                    {
                        "level": "P1",
                        "festival": "大暑",
                        "category": "节气"
                    }
                ],
                "2026-08-07": [
                    {
                        "level": "P0",
                        "festival": "立秋",
                        "category": "节气"
                    }
                ],
                "2026-08-23": [
                    {
                        "level": "P1",
                        "festival": "处暑",
                        "category": "节气"
                    }
                ],
                "2026-09-07": [
                    {
                        "level": "P1",
                        "festival": "白露",
                        "category": "节气"
                    },
                    {
                        "level": "P1",
                        "festival": "袁隆平诞辰纪念日",
                        "category": "纪念日"
                    }
                ],
                "2026-09-23": [
                    {
                        "level": "P1",
                        "festival": "秋分",
                        "category": "节气"
                    }
                ],
                "2026-10-08": [
                    {
                        "level": "P1",
                        "festival": "寒露",
                        "category": "节气"
                    }
                ],
                "2026-10-23": [
                    {
                        "level": "P1",
                        "festival": "霜降",
                        "category": "节气"
                    }
                ],
                "2026-11-07": [
                    {
                        "level": "P0",
                        "festival": "立冬",
                        "category": "节气"
                    }
                ],
                "2026-11-22": [
                    {
                        "level": "P1",
                        "festival": "小雪",
                        "category": "节气"
                    }
                ],
                "2026-12-07": [
                    {
                        "level": "P1",
                        "festival": "大雪",
                        "category": "节气"
                    }
                ],
                "2026-12-22": [
                    {
                        "level": "P0",
                        "festival": "冬至",
                        "category": "节气"
                    }
                ],
                "2026-11-12": [
                    {
                        "level": "P1",
                        "festival": "孙中山诞辰纪念日",
                        "category": "纪念日"
                    }
                ],
                "2026-12-26": [
                    {
                        "level": "P0",
                        "festival": "毛主席诞辰纪念日",
                        "category": "纪念日"
                    }
                ],
                "2026-09-09": [
                    {
                        "level": "P0",
                        "festival": "毛主席逝世纪念日",
                        "category": "纪念日"
                    }
                ],
                "2026-01-08": [
                    {
                        "level": "P0",
                        "festival": "周恩来逝世纪念日",
                        "category": "纪念日"
                    }
                ],
                "2026-08-22": [
                    {
                        "level": "P0",
                        "festival": "邓小平诞辰纪念日",
                        "category": "纪念日"
                    }
                ],
                "2026-07-03": [
                    {
                        "level": "P0",
                        "festival": "李克强诞辰纪念日",
                        "category": "纪念日"
                    }
                ],
                "2026-10-27": [
                    {
                        "level": "P0",
                        "festival": "李克强逝世纪念日",
                        "category": "纪念日"
                    },
                    {
                        "level": "P1",
                        "festival": "吴尊友逝世纪念日",
                        "category": "纪念日"
                    }
                ],
                "2026-09-18": [
                    {
                        "level": "P0",
                        "festival": "九一八纪念日",
                        "category": "纪念日"
                    }
                ],
                "2026-05-22": [
                    {
                        "level": "P1",
                        "festival": "袁隆平逝世纪念日",
                        "category": "纪念日"
                    }
                ],
                "2026-10-24": [
                    {
                        "level": "P1",
                        "festival": "彭德怀诞辰纪念日",
                        "category": "纪念日"
                    }
                ],
                "2026-11-29": [
                    {
                        "level": "P1",
                        "festival": "彭德怀逝世纪念日",
                        "category": "纪念日"
                    }
                ],
                "2026-12-01": [
                    {
                        "level": "P1",
                        "festival": "朱德诞辰纪念日",
                        "category": "纪念日"
                    }
                ],
                "2026-07-06": [
                    {
                        "level": "P1",
                        "festival": "朱德逝世纪念日",
                        "category": "纪念日"
                    }
                ]
            }
        }

    def date_checker(self):
        """
        check date of reminder
        :return:
        """
        year = datetime.datetime.today().strftime("%Y")
        today = datetime.datetime.today()
        tomorrow = today + datetime.timedelta(days=1)
        tomorrow = tomorrow.strftime("%Y-%m-%d")
        in_three_days = today + datetime.timedelta(days=3)
        in_three_days = in_three_days.strftime("%Y-%m-%d")
        if self.festival_dict[year].get(tomorrow):
            festival_list = self.festival_dict[year][tomorrow]
            for festival_ in festival_list:
                bot(festival_info=festival_, days_count=1, date_str=tomorrow)
                time.sleep(30)
        if self.festival_dict[year].get(in_three_days):
            festival_list = self.festival_dict[year][in_three_days]
            for festival_ in festival_list:
                print(festival_)
                bot(festival_info=festival_, days_count=3, date_str=in_three_days)
                time.sleep(30)

    def run(self):
        """
        执行定时任务
        :return:
        """
        schedule.every().day.at("10:00").do(self.date_checker)

        while True:
            schedule.run_pending()
            time.sleep(10)


if __name__ == "__main__":
    F = FestivalReminder()
    F.run()
